@@ -1,8 +1,11 @@
 use super::encode::BUFFER_SIZE;
 use crate::{metadata::MetadataValue, Status};
-use bytes::{Buf, BufMut, BytesMut};
+use bytes::{Buf, BytesMut};
+#[cfg(feature = "gzip")]
 use flate2::read::{GzDecoder, GzEncoder};
 use std::fmt;
+#[cfg(feature = "zstd")]
+use zstd::stream::read::{Decoder, Encoder};
 
 pub(crate) const ENCODING_HEADER: &str = "grpc-encoding";
 pub(crate) const ACCEPT_ENCODING_HEADER: &str = "grpc-accept-encoding";
@@ -10,28 +13,61 @@ pub(crate) const ACCEPT_ENCODING_HEADER: &str = "grpc-accept-encoding";
 /// Struct used to configure which encodings are enabled on a server or channel.
 #[derive(Debug, Default, Clone, Copy)]
 pub struct EnabledCompressionEncodings {
+    #[cfg(feature = "gzip")]
     pub(crate) gzip: bool,
+    #[cfg(feature = "zstd")]
+    pub(crate) zstd: bool,
 }
 
 impl EnabledCompressionEncodings {
-    /// Check if `gzip` compression is enabled.
-    pub fn gzip(self) -> bool {
-        self.gzip
+    /// Check if a [`CompressionEncoding`] is enabled.
+    pub fn is_enabled(&self, encoding: CompressionEncoding) -> bool {
+        match encoding {
+            #[cfg(feature = "gzip")]
+            CompressionEncoding::Gzip => self.gzip,
+            #[cfg(feature = "zstd")]
+            CompressionEncoding::Zstd => self.zstd,
+        }
     }
 
-    /// Enable `gzip` compression.
-    pub fn enable_gzip(&mut self) {
-        self.gzip = true;
+    /// Enable a [`CompressionEncoding`].
+    pub fn enable(&mut self, encoding: CompressionEncoding) {
+        match encoding {
+            #[cfg(feature = "gzip")]
+            CompressionEncoding::Gzip => self.gzip = true,
+            #[cfg(feature = "zstd")]
+            CompressionEncoding::Zstd => self.zstd = true,
+        }
     }
 
     pub(crate) fn into_accept_encoding_header_value(self) -> Option<http::HeaderValue> {
-        let Self { gzip } = self;
-        if gzip {
-            Some(http::HeaderValue::from_static("gzip,identity"))
-        } else {
-            None
+        match (self.is_gzip_enabled(), self.is_zstd_enabled()) {
+            (true, false) => Some(http::HeaderValue::from_static("gzip,identity")),
+            (false, true) => Some(http::HeaderValue::from_static("zstd,identity")),
+            (true, true) => Some(http::HeaderValue::from_static("gzip,zstd,identity")),
+            (false, false) => None,
         }
     }
+
+    #[cfg(feature = "gzip")]
+    const fn is_gzip_enabled(&self) -> bool {
+        self.gzip
+    }
+
+    #[cfg(not(feature = "gzip"))]
+    const fn is_gzip_enabled(&self) -> bool {
+        false
+    }
+
+    #[cfg(feature = "zstd")]
+    const fn is_zstd_enabled(&self) -> bool {
+        self.zstd
+    }
+
+    #[cfg(not(feature = "zstd"))]
+    const fn is_zstd_enabled(&self) -> bool {
+        false
+    }
 }
 
 /// The compression encodings Tonic supports.
@@ -39,7 +75,13 @@ impl EnabledCompressionEncodings {
 #[non_exhaustive]
 pub enum CompressionEncoding {
     #[allow(missing_docs)]
+    #[cfg(feature = "gzip")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "gzip")))]
     Gzip,
+    #[allow(missing_docs)]
+    #[cfg(feature = "zstd")]
+    #[cfg_attr(docsrs, doc(cfg(feature = "zstd")))]
+    Zstd,
 }
 
 impl CompressionEncoding {
@@ -48,13 +90,18 @@ impl CompressionEncoding {
         map: &http::HeaderMap,
         enabled_encodings: EnabledCompressionEncodings,
     ) -> Option<Self> {
+        if !enabled_encodings.is_gzip_enabled() && !enabled_encodings.is_zstd_enabled() {
+            return None;
+        }
+
         let header_value = map.get(ACCEPT_ENCODING_HEADER)?;
         let header_value_str = header_value.to_str().ok()?;
 
-        let EnabledCompressionEncodings { gzip } = enabled_encodings;
-
         split_by_comma(header_value_str).find_map(|value| match value {
-            "gzip" if gzip => Some(CompressionEncoding::Gzip),
+            #[cfg(feature = "gzip")]
+            "gzip" => Some(CompressionEncoding::Gzip),
+            #[cfg(feature = "zstd")]
+            "zstd" => Some(CompressionEncoding::Zstd),
             _ => None,
         })
     }
@@ -76,10 +123,15 @@ impl CompressionEncoding {
             return Ok(None);
         };
 
-        let EnabledCompressionEncodings { gzip } = enabled_encodings;
-
         match header_value_str {
-            "gzip" if gzip => Ok(Some(CompressionEncoding::Gzip)),
+            #[cfg(feature = "gzip")]
+            "gzip" if enabled_encodings.is_enabled(CompressionEncoding::Gzip) => {
+                Ok(Some(CompressionEncoding::Gzip))
+            }
+            #[cfg(feature = "zstd")]
+            "zstd" if enabled_encodings.is_enabled(CompressionEncoding::Zstd) => {
+                Ok(Some(CompressionEncoding::Zstd))
+            }
             "identity" => Ok(None),
             other => {
                 let mut status = Status::unimplemented(format!(
@@ -100,17 +152,40 @@ impl CompressionEncoding {
         }
     }
 
-    pub(crate) fn into_header_value(self) -> http::HeaderValue {
+    #[allow(missing_docs)]
+    #[cfg(any(feature = "gzip", feature = "zstd"))]
+    pub(crate) fn as_str(&self) -> &'static str {
         match self {
-            CompressionEncoding::Gzip => http::HeaderValue::from_static("gzip"),
+            #[cfg(feature = "gzip")]
+            CompressionEncoding::Gzip => "gzip",
+            #[cfg(feature = "zstd")]
+            CompressionEncoding::Zstd => "zstd",
         }
     }
+
+    #[cfg(any(feature = "gzip", feature = "zstd"))]
+    pub(crate) fn into_header_value(self) -> http::HeaderValue {
+        http::HeaderValue::from_static(self.as_str())
+    }
+
+    pub(crate) fn encodings() -> &'static [Self] {
+        &[
+            #[cfg(feature = "gzip")]
+            CompressionEncoding::Gzip,
+            #[cfg(feature = "zstd")]
+            CompressionEncoding::Zstd,
+        ]
+    }
 }
 
 impl fmt::Display for CompressionEncoding {
+    #[allow(unused_variables)]
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        match self {
+        match *self {
+            #[cfg(feature = "gzip")]
             CompressionEncoding::Gzip => write!(f, "gzip"),
+            #[cfg(feature = "zstd")]
+            CompressionEncoding::Zstd => write!(f, "zstd"),
         }
     }
 }
@@ -120,6 +195,7 @@ fn split_by_comma(s: &str) -> impl Iterator<Item = &str> {
 }
 
 /// Compress `len` bytes from `decompressed_buf` into `out_buf`.
+#[allow(unused_variables, unreachable_code)]
 pub(crate) fn compress(
     encoding: CompressionEncoding,
     decompressed_buf: &mut BytesMut,
@@ -129,17 +205,28 @@ pub(crate) fn compress(
     let capacity = ((len / BUFFER_SIZE) + 1) * BUFFER_SIZE;
     out_buf.reserve(capacity);
 
+    #[cfg(any(feature = "gzip", feature = "zstd"))]
+    let mut out_writer = bytes::BufMut::writer(out_buf);
+
     match encoding {
+        #[cfg(feature = "gzip")]
         CompressionEncoding::Gzip => {
             let mut gzip_encoder = GzEncoder::new(
                 &decompressed_buf[0..len],
                 // FIXME: support customizing the compression level
                 flate2::Compression::new(6),
             );
-            let mut out_writer = out_buf.writer();
-
             std::io::copy(&mut gzip_encoder, &mut out_writer)?;
         }
+        #[cfg(feature = "zstd")]
+        CompressionEncoding::Zstd => {
+            let mut zstd_encoder = Encoder::new(
+                &decompressed_buf[0..len],
+                // FIXME: support customizing the compression level
+                zstd::DEFAULT_COMPRESSION_LEVEL,
+            )?;
+            std::io::copy(&mut zstd_encoder, &mut out_writer)?;
+        }
     }
 
     decompressed_buf.advance(len);
@@ -148,6 +235,7 @@ pub(crate) fn compress(
 }
 
 /// Decompress `len` bytes from `compressed_buf` into `out_buf`.
+#[allow(unused_variables, unreachable_code)]
 pub(crate) fn decompress(
     encoding: CompressionEncoding,
     compressed_buf: &mut BytesMut,
@@ -158,13 +246,20 @@ pub(crate) fn decompress(
     let capacity = ((estimate_decompressed_len / BUFFER_SIZE) + 1) * BUFFER_SIZE;
     out_buf.reserve(capacity);
 
+    #[cfg(any(feature = "gzip", feature = "zstd"))]
+    let mut out_writer = bytes::BufMut::writer(out_buf);
+
     match encoding {
+        #[cfg(feature = "gzip")]
         CompressionEncoding::Gzip => {
             let mut gzip_decoder = GzDecoder::new(&compressed_buf[0..len]);
-            let mut out_writer = out_buf.writer();
-
             std::io::copy(&mut gzip_decoder, &mut out_writer)?;
         }
+        #[cfg(feature = "zstd")]
+        CompressionEncoding::Zstd => {
+            let mut zstd_decoder = Decoder::new(&compressed_buf[0..len])?;
+            std::io::copy(&mut zstd_decoder, &mut out_writer)?;
+        }
     }
 
     compressed_buf.advance(len);
