@@ -77,64 +77,8 @@ impl<VE: ValueEncoding> MetadataValue<VE> {
         }
     }
 
-    /// Attempt to convert a byte slice to a `MetadataValue`.
-    ///
-    /// For Ascii metadata values, If the argument contains invalid metadata
-    /// value bytes, an error is returned. Only byte values between 32 and 255
-    /// (inclusive) are permitted, excluding byte 127 (DEL).
-    ///
-    /// For Binary metadata values this method cannot fail. See also the Binary
-    /// only version of this method `from_bytes`.
-    ///
-    /// This function is intended to be replaced in the future by a `TryFrom`
-    /// implementation once the trait is stabilized in std.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// # use tonic::metadata::*;
-    /// let val = AsciiMetadataValue::try_from_bytes(b"hello\xfa").unwrap();
-    /// assert_eq!(val, &b"hello\xfa"[..]);
-    /// ```
-    ///
-    /// An invalid value
-    ///
-    /// ```
-    /// # use tonic::metadata::*;
-    /// let val = AsciiMetadataValue::try_from_bytes(b"\n");
-    /// assert!(val.is_err());
-    /// ```
-    #[inline]
-    pub fn try_from_bytes(src: &[u8]) -> Result<Self, InvalidMetadataValueBytes> {
-        VE::from_bytes(src).map(|value| MetadataValue {
-            inner: value,
-            phantom: PhantomData,
-        })
-    }
-
-    /// Attempt to convert a `Bytes` buffer to a `MetadataValue`.
-    ///
-    /// For `MetadataValue<Ascii>`, if the argument contains invalid metadata
-    /// value bytes, an error is returned. Only byte values between 32 and 255
-    /// (inclusive) are permitted, excluding byte 127 (DEL).
-    ///
-    /// For `MetadataValue<Binary>`, if the argument is not valid base64, an
-    /// error is returned. In use cases where the input is not base64 encoded,
-    /// use `from_bytes`; if the value has to be encoded it's not possible to
-    /// share the memory anyways.
-    ///
-    /// This function is intended to be replaced in the future by a `TryFrom`
-    /// implementation once the trait is stabilized in std.
-    #[inline]
-    pub fn from_shared(src: Bytes) -> Result<Self, InvalidMetadataValueBytes> {
-        VE::from_shared(src).map(|value| MetadataValue {
-            inner: value,
-            phantom: PhantomData,
-        })
-    }
-
     /// Convert a `Bytes` directly into a `MetadataValue` without validating.
-    /// For MetadataValue<Binary> the provided parameter must be base64
+    /// For `MetadataValue<Binary>` the provided parameter must be base64
     /// encoded without padding bytes at the end.
     ///
     /// # Safety
@@ -282,46 +226,165 @@ impl<VE: ValueEncoding> MetadataValue<VE> {
     }
 }
 
+/// Attempt to convert a byte slice to a `MetadataValue`.
+///
+/// For Ascii metadata values, If the argument contains invalid metadata
+/// value bytes, an error is returned. Only byte values between 32 and 255
+/// (inclusive) are permitted, excluding byte 127 (DEL).
+///
+/// For Binary metadata values this method cannot fail. See also the Binary
+/// only version of this method `from_bytes`.
+///
+/// # Examples
+///
+/// ```
+/// # use tonic::metadata::*;
+/// let val = AsciiMetadataValue::try_from(b"hello\xfa").unwrap();
+/// assert_eq!(val, &b"hello\xfa"[..]);
+/// ```
+///
+/// An invalid value
+///
+/// ```
+/// # use tonic::metadata::*;
+/// let val = AsciiMetadataValue::try_from(b"\n");
+/// assert!(val.is_err());
+/// ```
+impl<'a, VE: ValueEncoding> TryFrom<&'a [u8]> for MetadataValue<VE> {
+    type Error = InvalidMetadataValueBytes;
+
+    #[inline]
+    fn try_from(src: &[u8]) -> Result<Self, Self::Error> {
+        VE::from_bytes(src).map(|value| MetadataValue {
+            inner: value,
+            phantom: PhantomData,
+        })
+    }
+}
+
+/// Attempt to convert a byte slice to a `MetadataValue`.
+///
+/// For Ascii metadata values, If the argument contains invalid metadata
+/// value bytes, an error is returned. Only byte values between 32 and 255
+/// (inclusive) are permitted, excluding byte 127 (DEL).
+///
+/// For Binary metadata values this method cannot fail. See also the Binary
+/// only version of this method `from_bytes`.
+///
+/// # Examples
+///
+/// ```
+/// # use tonic::metadata::*;
+/// let val = AsciiMetadataValue::try_from(b"hello\xfa").unwrap();
+/// assert_eq!(val, &b"hello\xfa"[..]);
+/// ```
+///
+/// An invalid value
+///
+/// ```
+/// # use tonic::metadata::*;
+/// let val = AsciiMetadataValue::try_from(b"\n");
+/// assert!(val.is_err());
+/// ```
+impl<'a, VE: ValueEncoding, const N: usize> TryFrom<&'a [u8; N]> for MetadataValue<VE> {
+    type Error = InvalidMetadataValueBytes;
+
+    #[inline]
+    fn try_from(src: &[u8; N]) -> Result<Self, Self::Error> {
+        Self::try_from(src.as_ref())
+    }
+}
+
+/// Attempt to convert a `Bytes` buffer to a `MetadataValue`.
+///
+/// For `MetadataValue<Ascii>`, if the argument contains invalid metadata
+/// value bytes, an error is returned. Only byte values between 32 and 255
+/// (inclusive) are permitted, excluding byte 127 (DEL).
+///
+/// For `MetadataValue<Binary>`, if the argument is not valid base64, an
+/// error is returned. In use cases where the input is not base64 encoded,
+/// use `from_bytes`; if the value has to be encoded it's not possible to
+/// share the memory anyways.
+impl<VE: ValueEncoding> TryFrom<Bytes> for MetadataValue<VE> {
+    type Error = InvalidMetadataValueBytes;
+
+    #[inline]
+    fn try_from(src: Bytes) -> Result<Self, Self::Error> {
+        VE::from_shared(src).map(|value| MetadataValue {
+            inner: value,
+            phantom: PhantomData,
+        })
+    }
+}
+
+/// Attempt to convert a Vec of bytes to a `MetadataValue`.
+///
+/// For `MetadataValue<Ascii>`, if the argument contains invalid metadata
+/// value bytes, an error is returned. Only byte values between 32 and 255
+/// (inclusive) are permitted, excluding byte 127 (DEL).
+///
+/// For `MetadataValue<Binary>`, if the argument is not valid base64, an
+/// error is returned. In use cases where the input is not base64 encoded,
+/// use `from_bytes`; if the value has to be encoded it's not possible to
+/// share the memory anyways.
+impl<VE: ValueEncoding> TryFrom<Vec<u8>> for MetadataValue<VE> {
+    type Error = InvalidMetadataValueBytes;
+
+    #[inline]
+    fn try_from(src: Vec<u8>) -> Result<Self, Self::Error> {
+        Self::try_from(src.as_slice())
+    }
+}
+
+/// Attempt to convert a string to a `MetadataValue<Ascii>`.
+///
+/// If the argument contains invalid metadata value characters, an error is
+/// returned. Only visible ASCII characters (32-127) are permitted. Use
+/// `from_bytes` to create a `MetadataValue` that includes opaque octets
+/// (128-255).
+impl<'a> TryFrom<&'a str> for MetadataValue<Ascii> {
+    type Error = InvalidMetadataValue;
+
+    #[inline]
+    fn try_from(s: &'a str) -> Result<Self, Self::Error> {
+        s.parse()
+    }
+}
+
+/// Attempt to convert a string to a `MetadataValue<Ascii>`.
+///
+/// If the argument contains invalid metadata value characters, an error is
+/// returned. Only visible ASCII characters (32-127) are permitted. Use
+/// `from_bytes` to create a `MetadataValue` that includes opaque octets
+/// (128-255).
+impl<'a> TryFrom<&'a String> for MetadataValue<Ascii> {
+    type Error = InvalidMetadataValue;
+
+    #[inline]
+    fn try_from(s: &'a String) -> Result<Self, Self::Error> {
+        s.parse()
+    }
+}
+
+/// Attempt to convert a string to a `MetadataValue<Ascii>`.
+///
+/// If the argument contains invalid metadata value characters, an error is
+/// returned. Only visible ASCII characters (32-127) are permitted. Use
+/// `from_bytes` to create a `MetadataValue` that includes opaque octets
+/// (128-255).
+impl TryFrom<String> for MetadataValue<Ascii> {
+    type Error = InvalidMetadataValue;
+
+    #[inline]
+    fn try_from(s: String) -> Result<Self, Self::Error> {
+        s.parse()
+    }
+}
+
 // is_empty is defined in the generic impl block above
 #[allow(clippy::len_without_is_empty)]
 impl MetadataValue<Ascii> {
-    /// Attempt to convert a string to a `MetadataValue<Ascii>`.
-    ///
-    /// If the argument contains invalid metadata value characters, an error is
-    /// returned. Only visible ASCII characters (32-127) are permitted. Use
-    /// `from_bytes` to create a `MetadataValue` that includes opaque octets
-    /// (128-255).
-    ///
-    /// This function is intended to be replaced in the future by a `TryFrom`
-    /// implementation once the trait is stabilized in std.
-    ///
-    /// # Examples
-    ///
-    /// ```
-    /// # use tonic::metadata::*;
-    /// let val = AsciiMetadataValue::from_str("hello").unwrap();
-    /// assert_eq!(val, "hello");
-    /// ```
-    ///
-    /// An invalid value
-    ///
-    /// ```
-    /// # use tonic::metadata::*;
-    /// let val = AsciiMetadataValue::from_str("\n");
-    /// assert!(val.is_err());
-    /// ```
-    #[allow(clippy::should_implement_trait)]
-    #[inline]
-    pub fn from_str(src: &str) -> Result<Self, InvalidMetadataValue> {
-        HeaderValue::from_str(src)
-            .map(|value| MetadataValue {
-                inner: value,
-                phantom: PhantomData,
-            })
-            .map_err(|_| InvalidMetadataValue::new())
-    }
-
-    /// Converts a MetadataKey into a MetadataValue<Ascii>.
+    /// Converts a MetadataKey into a `MetadataValue<Ascii>`.
     ///
     /// Since every valid MetadataKey is a valid MetadataValue this is done
     /// infallibly.
@@ -331,7 +394,7 @@ impl MetadataValue<Ascii> {
     /// ```
     /// # use tonic::metadata::*;
     /// let val = AsciiMetadataValue::from_key::<Ascii>("accept".parse().unwrap());
-    /// assert_eq!(val, AsciiMetadataValue::try_from_bytes(b"accept").unwrap());
+    /// assert_eq!(val, AsciiMetadataValue::try_from(b"accept").unwrap());
     /// ```
     #[inline]
     pub fn from_key<KeyVE: ValueEncoding>(key: MetadataKey<KeyVE>) -> Self {
@@ -340,9 +403,9 @@ impl MetadataValue<Ascii> {
 
     /// Returns the length of `self`, in bytes.
     ///
-    /// This method is not available for MetadataValue<Binary> because that
+    /// This method is not available for `MetadataValue<Binary>` because that
     /// cannot be implemented in constant time, which most people would probably
-    /// expect. To get the length of MetadataValue<Binary>, convert it to a
+    /// expect. To get the length of `MetadataValue<Binary>`, convert it to a
     /// Bytes value and measure its length.
     ///
     /// # Examples
@@ -402,8 +465,8 @@ impl MetadataValue<Binary> {
     /// ```
     #[inline]
     pub fn from_bytes(src: &[u8]) -> Self {
-        // Only the Ascii version of try_from_bytes can fail.
-        Self::try_from_bytes(src).unwrap()
+        // Only the Ascii version of try_from can fail.
+        Self::try_from(src).unwrap()
     }
 }
 
@@ -501,7 +564,7 @@ mod from_metadata_value_tests {
 
         assert_eq!(
             map.get("accept").unwrap(),
-            AsciiMetadataValue::try_from_bytes(b"hello-world").unwrap()
+            AsciiMetadataValue::try_from(b"hello-world").unwrap()
         );
     }
 }
@@ -511,7 +574,12 @@ impl FromStr for MetadataValue<Ascii> {
 
     #[inline]
     fn from_str(s: &str) -> Result<MetadataValue<Ascii>, Self::Err> {
-        MetadataValue::<Ascii>::from_str(s)
+        HeaderValue::from_str(s)
+            .map(|value| MetadataValue {
+                inner: value,
+                phantom: PhantomData,
+            })
+            .map_err(|_| InvalidMetadataValue::new())
     }
 }
 
@@ -578,7 +646,7 @@ impl<VE: ValueEncoding> Eq for MetadataValue<VE> {}
 impl<VE: ValueEncoding> PartialOrd for MetadataValue<VE> {
     #[inline]
     fn partial_cmp(&self, other: &MetadataValue<VE>) -> Option<cmp::Ordering> {
-        self.inner.partial_cmp(&other.inner)
+        Some(self.cmp(other))
     }
 }
 
@@ -730,7 +798,7 @@ fn test_debug() {
     ];
 
     for &(value, expected) in cases {
-        let val = AsciiMetadataValue::try_from_bytes(value.as_bytes()).unwrap();
+        let val = AsciiMetadataValue::try_from(value.as_bytes()).unwrap();
         let actual = format!("{:?}", val);
         assert_eq!(expected, actual);
     }
@@ -760,7 +828,7 @@ fn test_is_empty() {
 
 #[test]
 fn test_from_shared_base64_encodes() {
-    let value = BinaryMetadataValue::from_shared(Bytes::from_static(b"Hello")).unwrap();
+    let value = BinaryMetadataValue::try_from(Bytes::from_static(b"Hello")).unwrap();
     assert_eq!(value.as_encoded_bytes(), b"SGVsbG8");
 }
 
