@@ -1,9 +1,10 @@
 use crate::body::BoxBody;
 use crate::metadata::MetadataMap;
+use base64::Engine as _;
 use bytes::Bytes;
 use http::header::{HeaderMap, HeaderValue};
 use percent_encoding::{percent_decode, percent_encode, AsciiSet, CONTROLS};
-use std::{borrow::Cow, error::Error, fmt};
+use std::{borrow::Cow, error::Error, fmt, sync::Arc};
 use tracing::{debug, trace, warn};
 
 const ENCODING_SET: &AsciiSet = &CONTROLS
@@ -33,6 +34,7 @@ const GRPC_STATUS_DETAILS_HEADER: &str = "grpc-status-details-bin";
 /// assert_eq!(status1.code(), Code::InvalidArgument);
 /// assert_eq!(status1.code(), status2.code());
 /// ```
+#[derive(Clone)]
 pub struct Status {
     /// The gRPC status code, found in the `grpc-status` header.
     code: Code,
@@ -45,7 +47,7 @@ pub struct Status {
     /// or by `Status` fields above, they will be ignored.
     metadata: MetadataMap,
     /// Optional underlying error.
-    source: Option<Box<dyn Error + Send + Sync + 'static>>,
+    source: Option<Arc<dyn Error + Send + Sync + 'static>>,
 }
 
 /// gRPC status codes used by [`Status`].
@@ -304,15 +306,33 @@ impl Status {
     }
 
     #[cfg_attr(not(feature = "transport"), allow(dead_code))]
-    pub(crate) fn from_error(err: Box<dyn Error + Send + Sync + 'static>) -> Status {
+    pub(crate) fn from_error_generic(
+        err: impl Into<Box<dyn Error + Send + Sync + 'static>>,
+    ) -> Status {
+        Self::from_error(err.into())
+    }
+
+    /// Create a `Status` from various types of `Error`.
+    ///
+    /// Inspects the error source chain for recognizable errors, including statuses, HTTP2, and
+    /// hyper, and attempts to maps them to a `Status`, or else returns an Unknown `Status`.
+    #[cfg_attr(not(feature = "transport"), allow(dead_code))]
+    pub fn from_error(err: Box<dyn Error + Send + Sync + 'static>) -> Status {
         Status::try_from_error(err).unwrap_or_else(|err| {
             let mut status = Status::new(Code::Unknown, err.to_string());
-            status.source = Some(err);
+            status.source = Some(err.into());
             status
         })
     }
 
-    pub(crate) fn try_from_error(
+    /// Create a `Status` from various types of `Error`.
+    ///
+    /// Returns the error if a status could not be created.
+    ///
+    /// # Downcast stability
+    /// This function does not provide any stability guarantees around how it will downcast errors into
+    /// status codes.
+    pub fn try_from_error(
         err: Box<dyn Error + Send + Sync + 'static>,
     ) -> Result<Status, Box<dyn Error + Send + Sync + 'static>> {
         let err = match err.downcast::<Status>() {
@@ -331,7 +351,7 @@ impl Status {
         };
 
         if let Some(mut status) = find_status_in_source_chain(&*err) {
-            status.source = Some(err);
+            status.source = Some(err.into());
             return Ok(status);
         }
 
@@ -341,8 +361,17 @@ impl Status {
     // FIXME: bubble this into `transport` and expose generic http2 reasons.
     #[cfg(feature = "transport")]
     fn from_h2_error(err: Box<h2::Error>) -> Status {
+        let code = Self::code_from_h2(&err);
+
+        let mut status = Self::new(code, format!("h2 protocol error: {}", err));
+        status.source = Some(Arc::new(*err));
+        status
+    }
+
+    #[cfg(feature = "transport")]
+    fn code_from_h2(err: &h2::Error) -> Code {
         // See https://github.com/grpc/grpc/blob/3977c30/doc/PROTOCOL-HTTP2.md#errors
-        let code = match err.reason() {
+        match err.reason() {
             Some(h2::Reason::NO_ERROR)
             | Some(h2::Reason::PROTOCOL_ERROR)
             | Some(h2::Reason::INTERNAL_ERROR)
@@ -356,11 +385,7 @@ impl Status {
             Some(h2::Reason::INADEQUATE_SECURITY) => Code::PermissionDenied,
 
             _ => Code::Unknown,
-        };
-
-        let mut status = Self::new(code, format!("h2 protocol error: {}", err));
-        status.source = Some(err);
-        status
+        }
     }
 
     #[cfg(feature = "transport")]
@@ -396,6 +421,14 @@ impl Status {
         if err.is_timeout() || err.is_connect() {
             return Some(Status::unavailable(err.to_string()));
         }
+
+        if let Some(h2_err) = err.source().and_then(|e| e.downcast_ref::<h2::Error>()) {
+            let code = Status::code_from_h2(h2_err);
+            let status = Self::new(code, format!("h2 protocol error: {}", err));
+
+            return Some(status);
+        }
+
         None
     }
 
@@ -423,11 +456,12 @@ impl Status {
             let details = header_map
                 .get(GRPC_STATUS_DETAILS_HEADER)
                 .map(|h| {
-                    base64::decode(h.as_bytes())
+                    crate::util::base64::STANDARD
+                        .decode(h.as_bytes())
                         .expect("Invalid status header, expected base64 encoded value")
                 })
                 .map(Bytes::from)
-                .unwrap_or_else(Bytes::new);
+                .unwrap_or_default();
 
             let mut other_headers = header_map.clone();
             other_headers.remove(GRPC_STATUS_HEADER_CODE);
@@ -487,7 +521,8 @@ impl Status {
         Ok(header_map)
     }
 
-    pub(crate) fn add_header(&self, header_map: &mut HeaderMap) -> Result<(), Self> {
+    /// Add headers from this `Status` into `header_map`.
+    pub fn add_header(&self, header_map: &mut HeaderMap) -> Result<(), Self> {
         header_map.extend(self.metadata.clone().into_sanitized_headers());
 
         header_map.insert(GRPC_STATUS_HEADER_CODE, self.code.to_header_value());
@@ -504,7 +539,7 @@ impl Status {
         }
 
         if !self.details.is_empty() {
-            let details = base64::encode_config(&self.details[..], base64::STANDARD_NO_PAD);
+            let details = crate::util::base64::STANDARD_NO_PAD.encode(&self.details[..]);
 
             header_map.insert(
                 GRPC_STATUS_DETAILS_HEADER,
@@ -541,6 +576,12 @@ impl Status {
         }
     }
 
+    /// Add a source error to this status.
+    pub fn set_source(&mut self, source: Arc<dyn Error + Send + Sync + 'static>) -> &mut Status {
+        self.source = Some(source);
+        self
+    }
+
     #[allow(clippy::wrong_self_convention)]
     /// Build an `http::Response` from the given `Status`.
     pub fn to_http(self) -> http::Response<BoxBody> {
@@ -959,7 +1000,7 @@ mod tests {
 
         let header_map = status.to_header_map().unwrap();
 
-        let b64_details = base64::encode_config(DETAILS, base64::STANDARD_NO_PAD);
+        let b64_details = crate::util::base64::STANDARD_NO_PAD.encode(DETAILS);
 
         assert_eq!(header_map[super::GRPC_STATUS_DETAILS_HEADER], b64_details);
 
