@@ -18,6 +18,7 @@
 //! ## Client
 //!
 //! ```no_run
+//! # #[cfg(feature = "rustls")]
 //! # use tonic::transport::{Channel, Certificate, ClientTlsConfig};
 //! # use std::time::Duration;
 //! # use tonic::body::BoxBody;
@@ -45,9 +46,9 @@
 //! ## Server
 //!
 //! ```no_run
+//! # #[cfg(feature = "rustls")]
 //! # use tonic::transport::{Server, Identity, ServerTlsConfig};
-//! # use tower::{Service, service_fn};
-//! # use futures_util::future::{err, ok};
+//! # use tower::Service;
 //! # #[cfg(feature = "rustls")]
 //! # async fn do_thing() -> Result<(), Box<dyn std::error::Error>> {
 //! # #[derive(Clone)]
@@ -55,7 +56,7 @@
 //! # impl Service<hyper::Request<hyper::Body>> for Svc {
 //! #   type Response = hyper::Response<tonic::body::BoxBody>;
 //! #   type Error = tonic::Status;
-//! #   type Future = futures_util::future::Ready<Result<Self::Response, Self::Error>>;
+//! #   type Future = std::future::Ready<Result<Self::Response, Self::Error>>;
 //! #   fn poll_ready(&mut self, _cx: &mut std::task::Context<'_>) -> std::task::Poll<Result<(), Self::Error>> {
 //! #       Ok(()).into()
 //! #  }
@@ -63,7 +64,7 @@
 //! #       unimplemented!()
 //! #   }
 //! # }
-//! # impl tonic::transport::NamedService for Svc {
+//! # impl tonic::server::NamedService for Svc {
 //! # const NAME: &'static str = "some_svc";
 //! # }
 //! # let my_svc = Svc;
@@ -91,16 +92,22 @@ pub mod server;
 
 mod error;
 mod service;
+#[cfg(feature = "tls")]
 mod tls;
 
 #[doc(inline)]
+#[cfg(feature = "channel")]
+#[cfg_attr(docsrs, doc(cfg(feature = "channel")))]
 pub use self::channel::{Channel, Endpoint};
 pub use self::error::Error;
 #[doc(inline)]
-pub use self::server::{NamedService, Server};
+pub use self::server::Server;
 #[doc(inline)]
 pub use self::service::grpc_timeout::TimeoutExpired;
+#[cfg(feature = "tls")]
+#[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 pub use self::tls::Certificate;
+pub use axum::{body::BoxBody as AxumBoxBody, Router as AxumRouter};
 pub use hyper::{Body, Uri};
 
 pub(crate) use self::service::executor::Executor;
@@ -115,5 +122,4 @@ pub use self::server::ServerTlsConfig;
 #[cfg_attr(docsrs, doc(cfg(feature = "tls")))]
 pub use self::tls::Identity;
 
-type BoxFuture<T, E> =
-    std::pin::Pin<Box<dyn std::future::Future<Output = Result<T, E>> + Send + 'static>>;
+type BoxFuture<'a, T> = std::pin::Pin<Box<dyn std::future::Future<Output = T> + Send + 'a>>;
