@@ -3,23 +3,11 @@ use super::io::BoxedIo;
 #[cfg(feature = "tls")]
 use super::tls::TlsConnector;
 use http::Uri;
-#[cfg(feature = "tls-roots-common")]
-use std::convert::TryInto;
 use std::fmt;
 use std::task::{Context, Poll};
 use tower::make::MakeConnection;
 use tower_service::Service;
 
-#[cfg(not(feature = "tls"))]
-pub(crate) fn connector<C>(inner: C) -> Connector<C> {
-    Connector::new(inner)
-}
-
-#[cfg(feature = "tls")]
-pub(crate) fn connector<C>(inner: C, tls: Option<TlsConnector>) -> Connector<C> {
-    Connector::new(inner, tls)
-}
-
 pub(crate) struct Connector<C> {
     inner: C,
     #[cfg(feature = "tls")]
@@ -36,7 +24,7 @@ impl<C> Connector<C> {
     }
 
     #[cfg(feature = "tls")]
-    fn new(inner: C, tls: Option<TlsConnector>) -> Self {
+    pub(crate) fn new(inner: C, tls: Option<TlsConnector>) -> Self {
         Self { inner, tls }
     }
 
@@ -51,9 +39,7 @@ impl<C> Connector<C> {
             _ => return None,
         };
 
-        host.try_into()
-            .ok()
-            .and_then(|dns| TlsConnector::new(None, None, dns).ok())
+        TlsConnector::new(None, None, host).ok()
     }
 }
 
@@ -66,7 +52,7 @@ where
 {
     type Response = BoxedIo;
     type Error = crate::Error;
-    type Future = BoxFuture<Self::Response, Self::Error>;
+    type Future = BoxFuture<'static, Result<Self::Response, Self::Error>>;
 
     fn poll_ready(&mut self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
         MakeConnection::poll_ready(&mut self.inner, cx).map_err(Into::into)
@@ -79,24 +65,22 @@ where
         #[cfg(feature = "tls-roots-common")]
         let tls = self.tls_or_default(uri.scheme_str(), uri.host());
 
+        #[cfg(feature = "tls")]
         let is_https = uri.scheme_str() == Some("https");
         let connect = self.inner.make_connection(uri);
 
         Box::pin(async move {
-            #[cfg(not(feature = "tls"))]
-            {
-                if is_https {
-                    return Err(HttpsUriWithoutTlsSupport(()).into());
-                }
-            }
-
             let io = connect.await?;
 
             #[cfg(feature = "tls")]
             {
                 if let Some(tls) = tls {
-                    let conn = tls.connect(io).await?;
-                    return Ok(BoxedIo::new(conn));
+                    if is_https {
+                        let conn = tls.connect(io).await?;
+                        return Ok(BoxedIo::new(conn));
+                    } else {
+                        return Ok(BoxedIo::new(io));
+                    }
                 } else if is_https {
                     return Err(HttpsUriWithoutTlsSupport(()).into());
                 }
